@@ -234,7 +234,7 @@ let conflict_report incpath pkglist =
   (* Second check whether there are module conflicts *)
   let pkgpath =
     List.map Findlib.package_directory pkglist in
-  Fl_package_base.module_conflict_report ~identify_dir incpath;
+  Fl_package_base.module_conflict_report ~identify_dir (pkgpath @ incpath);
 
   (* Finally check whether there are multiple DLLs: *)
   (* Note: Only the directories mentioned in ld.conf are checked, but not the
@@ -373,6 +373,46 @@ let run_command ?filter verbose cmd args =
 ;;
 
 
+(**************** Generic argument processing *************************)
+
+let merge_native_arguments native_spec f_unit f_string f_special_list =
+  List.map
+    (fun (switch_name, switch_has_arg, help_text) ->
+       let f =
+	 try
+	   List.assoc switch_name f_special_list
+	 with
+	     Not_found ->
+	       if switch_has_arg then 
+		 f_string switch_name 
+	       else 
+		 f_unit switch_name in
+       (switch_name, f, help_text)
+    )
+    native_spec
+;;
+
+
+let parse_args
+      ?(current = Arg.current) ?(args = Sys.argv) 
+      ?(align = true)
+      spec anon usage =
+  try
+    Arg.parse_argv
+      ~current
+      args
+      (if align then Arg.align spec else spec)
+      anon
+      usage
+  with
+    | Arg.Help text ->
+        print_string text;
+        exit 0
+    | Arg.Bad text ->
+        prerr_string text;
+        exit 2  
+
+
 (************************* format expansion *************************)
 
 
@@ -461,17 +501,17 @@ let query_package () =
   in
 
 
-  Arg.parse
+  parse_args
     [ "-predicates", Arg.String append_predicate,
                   "      specifies comma-separated list of assumed predicates";
       "-format", Arg.String (fun s -> format := s),
-              "          specifies the output format";
+              "<fmt>      specifies the output format";
       "-separator", Arg.String (fun s -> separator := s),
                  "       specifies the string that separates multiple answers";
       "-prefix", Arg.String (fun s -> prefix := s),
-              "          a string printed before the first answer";
+              "<p>        a string printed before the first answer";
       "-suffix", Arg.String (fun s -> suffix := s),
-              "          a string printed after the last answer";
+              "<s>        a string printed after the last answer";
       "-recursive", Arg.Set recursive,
                  "       select direct and indirect ancestors/descendants, too";
       "-r", Arg.Set recursive,
@@ -584,16 +624,18 @@ let process_pp_spec syntax_preds packages pp_opts =
     if syntax_preds <> [] then
       match preprocessor_cmds with
 	  [] ->
-	    failwith("When using -syntax, the META variable 'preprocessor' must be set")
+	    failwith("Using -syntax, but no package is selected specifying \
+                     a preprocessor as required for -syntax")
 	| [_, cmd] -> Some cmd
 	| _ ->
-	    failwith("No unique value for the META variable 'preprocessor': " ^
-		     String.concat ", "
-		       (List.map
-			  (fun (n,v) ->
-			  "package " ^ n ^ " defines `" ^ v ^ "'")
-			  preprocessor_cmds
-		       )
+	    failwith("Several packages are selected that specify \
+                      preprocessors: " ^ 
+		       String.concat ", "
+		      (List.map
+			 (fun (n,v) ->
+			    "package " ^ n ^ " defines `" ^ v ^ "'")
+			 preprocessor_cmds
+		      )
 		    )
     else
       None
@@ -633,26 +675,6 @@ let process_pp_spec syntax_preds packages pp_opts =
 ;;
 
 
-(**************** Generic argument processing *************************)
-
-let merge_native_arguments native_spec f_unit f_string f_special_list =
-  List.map
-    (fun (switch_name, switch_has_arg, help_text) ->
-       let f =
-	 try
-	   List.assoc switch_name f_special_list
-	 with
-	     Not_found ->
-	       if switch_has_arg then 
-		 f_string switch_name 
-	       else 
-		 f_unit switch_name in
-       (switch_name, f, help_text)
-    )
-    native_spec
-;;
-
-
 (**************** OCAMLC/OCAMLMKTOP/OCAMLOPT subcommands ****************)
 
 type pass_file_t =
@@ -663,10 +685,17 @@ type pass_file_t =
 ;;
 
 
+let contracted_ocamlmklib_options =
+  [ "-l"; "-L"; "-R"; "-F"; "-Wl,-rpath,"; "-Wl,-R" ]
+    (* The ocamlmklib options where the argument is directly attached to the
+       switch (e.g. -L<path> instead of -L <path>)
+     *)
+
+
 let ocamlc which () =
 
-  let destdir = ref (default_location()) in
-
+  (* let destdir = ref (default_location()) in *)
+  
   let switches = ref [] in
   let pass_options = ref [] in
   let pass_files = ref [] in
@@ -703,7 +732,12 @@ let ocamlc which () =
                 pass_options := !pass_options @ [name]) in
   let add_spec_fn name s =
     pass_options := !pass_options @ [name; s] in
-  let add_spec name = Arg.String (add_spec_fn name) in
+  let add_spec name = 
+    Arg.String (add_spec_fn name) in
+  let add_contracted_spec_fn name s =
+    pass_options := !pass_options @ [name ^ s] in
+  let add_contracted_spec name =
+    Arg.String (add_contracted_spec_fn name) in
   let add_pkg =
     Arg.String (fun s -> packages := !packages @ (Fl_split.in_words s)) in
   let add_pred =
@@ -725,80 +759,118 @@ let ocamlc which () =
       | "ocamlmklib" -> Ocaml_args.ocamlmklib_spec
       | "ocamlmktop" -> Ocaml_args.ocamlmktop_spec
       | "ocamlopt"   -> Ocaml_args.ocamlopt_spec
+      | "ocamloptp"  -> Ocaml_args.ocamloptp_spec
       | _            -> None in
   let native_spec =
     match native_spec_opt with
       | None -> failwith ("Not supported in your configuration: " ^ which)
       | Some s -> s in
 
-  Arg.parse
-    (List.flatten
-    [ [
-      "-package", add_pkg,
-               " <name>   Refer to package when compiling";
-      "-linkpkg", Arg.Set linkpkg,
-               "          Link the packages in";
-      "-predicates", add_pred,
-                  " <p>   Add predicate <p> when resolving package properties";
-      "-dontlink", add_dontlink,
-                " <name>  Do not link in package <name> and its ancestors";
-      "-syntax", add_syntax_pred,
-              " <p>       Use preprocessor with predicate <p>";
-      "-ppopt", add_pp_opt,
-             " <opt>      Append option <opt> to preprocessor invocation";
-      "-dllpath-pkg", add_dll_pkg,
-                   "<pkg> Add -dllpath for this package";
-      "-dllpath-all", Arg.Set dll_pkgs_all,
-                   "      Add -dllpath for all linked packages";
-      "-ignore-error", Arg.Set ignore_error,
-                    "     Ignore the 'error' directive in META files";
-      "-passopt", Arg.String (fun s -> pass_options := !pass_options @ [s]),
-               " <opt>    Pass option <opt> directly to ocamlc/opt/mklib/mktop\nSTANDARD OPTIONS:";
-      ];
+  let arg_spec =
+    List.flatten
+      [ [
+          "-package", add_pkg,
+            "<name>   Refer to package when compiling";
+          "-linkpkg", Arg.Set linkpkg,
+            "          Link the packages in";
+          "-predicates", add_pred,
+            "<p>   Add predicate <p> when resolving package properties";
+          "-dontlink", add_dontlink,
+            "<name>  Do not link in package <name> and its ancestors";
+          "-syntax", add_syntax_pred,
+            "<p>       Use preprocessor with predicate <p>";
+          "-ppopt", add_pp_opt,
+            "<opt>      Append option <opt> to preprocessor invocation";
+          "-dllpath-pkg", add_dll_pkg,
+            "<pkg> Add -dllpath for this package";
+          "-dllpath-all", Arg.Set dll_pkgs_all,
+            "      Add -dllpath for all linked packages";
+          "-ignore-error", Arg.Set ignore_error,
+            "     Ignore the 'error' directive in META files";
+          "-passopt", Arg.String (fun s -> pass_options := !pass_options @ [s]),
+            "<opt>    Pass option <opt> directly to ocamlc/opt/mklib/mktop";
+          "-passrest", Arg.Rest (fun s -> pass_options := !pass_options @ [s]),
+            "         Pass all remaining options directly\nSTANDARD OPTIONS:";
+        ];
 
-      merge_native_arguments 
-	native_spec 
-	add_switch
-	add_spec
-	[ "-cclib", 
-	  Arg.String (fun s -> pass_files := !pass_files @ [ Cclib s ]);
+        merge_native_arguments 
+	  native_spec 
+	  add_switch
+	  add_spec
+          (
+	    [ "-cclib", 
+	      Arg.String (fun s -> pass_files := !pass_files @ [ Cclib s ]);
+              
+	      "-I", (Arg.String
+		       (fun s ->
+		          let s = resolve_path s in
+                          if Sys.file_exists s then incpath := s :: !incpath;  (* reverted below *)
+		          add_spec_fn "-I" (slashify s) ));
+              
+	      "-impl", 
+	      Arg.String (fun s -> pass_files := !pass_files @ [ Impl(slashify s) ]);
+              
+	      "-intf", 
+	      Arg.String (fun s -> pass_files := !pass_files @ [ Intf(slashify s) ]);
+              
+	      "-pp", 
+	      Arg.String (fun s -> pp_specified := true; add_spec_fn "-pp" s);
+	      
+	      "-thread", 
+	      Arg.Unit (fun _ -> threads := threads_default);
+            
+	      "-vmthread", 
+	      Arg.Unit (fun _ -> threads := `VM_threads);
+              
+	      "-", 
+	      Arg.String (fun s -> pass_files := !pass_files @  [ Pass s ]);
+              
+	    ] @
+            if which = "ocamlmklib" then
+              List.map
+                (fun opt ->
+                   (opt, add_contracted_spec opt)
+                )
+                contracted_ocamlmklib_options
+            else
+              []
+          )
+      ] in
 
-	  "-I", (Arg.String
-		   (fun s ->
-		      let s = resolve_path s in
-                      if Sys.file_exists s then incpath := s :: !incpath;  (* reverted below *)
-		      add_spec_fn "-I" (slashify s) ));
+  let (current,args) =
+    if which = "ocamlmklib" then
+      (* Special processing for -L, -R etc. *)
+      let c = !(Arg.current) in
+      let l = Array.length Sys.argv in
+      let args1 = Array.sub Sys.argv (c+1) (l-c-1) in
+      let args2 =
+        Array.append
+          [| Sys.argv.(0) |]
+          (Fl_args.rewrite_contracted_args
+             arg_spec
+             contracted_ocamlmklib_options
+             args1
+          ) in
+      (ref 0, args2)
+    else
+      (Arg.current, Sys.argv) in
 
-	  "-impl", 
-	  Arg.String (fun s -> pass_files := !pass_files @ [ Impl(slashify s) ]);
-
-	  "-intf", 
-	  Arg.String (fun s -> pass_files := !pass_files @ [ Intf(slashify s) ]);
-
-	  "-pp", 
-	  Arg.String (fun s -> pp_specified := true; add_spec_fn "-pp" s);
-	  
-	  "-thread", 
-	  Arg.Unit (fun _ -> threads := threads_default);
-
-	  "-vmthread", 
-	  Arg.Unit (fun _ -> threads := `VM_threads);
-
-	  "-", 
-	  Arg.String (fun s -> pass_files := !pass_files @  [ Pass s ]);
-	]
-    ])
+  parse_args
+    ~current
+    ~args
+    arg_spec
     (fun s -> pass_files := !pass_files @ [ Pass s])
     ("usage: ocamlfind " ^ which ^ " [options] file ...");
 
   (* ---- Start requirements analysis ---- *)
-
+  
   begin match which with
     "ocamlc"     -> predicates := "byte" :: !predicates;
   | "ocamlcp"    -> predicates := "byte" :: !predicates;
   | "ocamlmklib" -> predicates := "byte" :: "native" :: !predicates;
   | "ocamlmktop" -> predicates := "byte" :: "create_toploop" :: !predicates;
   | "ocamlopt"   -> predicates := "native" :: !predicates;
+  | "ocamloptp"  -> predicates := "native" :: !predicates;
   | _            -> failwith "unsupported backend"
   end;
 
@@ -850,7 +922,7 @@ let ocamlc which () =
   let eff_link =
     List.flatten
       (List.map
-	 (fun pkg -> if List.mem pkg !dontlink then [] else [pkg])
+	 (fun pkg -> if List.mem pkg eff_dontlink then [] else [pkg])
 	 eff_packages) in
 
 
@@ -1044,7 +1116,7 @@ let ocamlc which () =
       (List.map
 	 (function
 	      Pass s ->
-		if s.[0] = '-'
+		if s <> "" && s.[0] = '-'
 		then [ "-"; String.sub s 1 (String.length s - 1) ]
 		else [ resolve_path s ]
 	    | Impl s ->
@@ -1091,6 +1163,7 @@ let ocamlc which () =
       | "ocamlcp"    -> Findlib.command `ocamlcp
       | "ocamlmklib" -> Findlib.command `ocamlmklib
       | "ocamlmktop" -> Findlib.command `ocamlmktop
+      | "ocamloptp"  -> Findlib.command `ocamloptp
       | _            -> assert false
   in
 
@@ -1117,38 +1190,48 @@ let ocamldoc() =
       | None -> failwith "Not supported in your configuration: ocamldoc"
       | Some s -> s in
 
-  Arg.parse
-    ( [ "-package",
-	Arg.String (fun s -> 
-		      packages := Fl_split.in_words s @ !packages),
-	"<name>  Add this package to the search path";
+  parse_args
+    ~align:false
+    ( Arg.align
+        [ "-package",
+	  Arg.String (fun s -> 
+		        packages := Fl_split.in_words s @ !packages),
+	  "<name>  Add this package to the search path";
+          
+	  "-predicates",
+	  Arg.String (fun s ->
+		        predicates := Fl_split.in_words s @ !predicates),
+	  "<p>  Add predicate <p> when calculating dependencies";
 
-	"-predicates",
-	Arg.String (fun s ->
-		      predicates := Fl_split.in_words s @ !predicates),
-	"<p>  Add predicate <p> when calculating dependencies";
+	  "-syntax",
+	  Arg.String (fun s ->
+		        syntax_preds := Fl_split.in_words s @ !syntax_preds),
+	  "<p>  Use preprocessor with predicate <p>";
 
-	"-syntax",
-	Arg.String (fun s ->
-		      syntax_preds := Fl_split.in_words s @ !syntax_preds),
-	"<p>      Use preprocessor with predicate <p>";
-
-	"-ppopt",
-	Arg.String (fun s -> pp_opts := s :: !pp_opts),
-	"<opt>     Append option <opt> to preprocessor invocation";
-
-	"-thread",
-	Arg.Unit (fun () -> predicates := "mt" :: "mt_posix" :: !predicates),
-	"   Assume kernel multi-threading when doing dependency analyses";
-
-	"-vmthread",
-	Arg.Unit (fun () -> predicates := "mt" :: "mt_vm" :: !predicates),
-	"   Assume bytecode multi-threading when doing dependency analyses";
-
-	"-verbose",
-	Arg.Set verbose,
-	"        Be verbose\nSTANDARD OPTIONS:";
-      ]
+	  "-ppopt",
+	  Arg.String (fun s -> pp_opts := s :: !pp_opts),
+	  "<opt>  Append option <opt> to preprocessor invocation";
+          
+	  "-thread",
+	  Arg.Unit (fun () -> predicates := "mt" :: "mt_posix" :: !predicates),
+	  "  Assume kernel multi-threading when doing dependency analyses";
+          
+	  "-vmthread",
+	  Arg.Unit (fun () -> predicates := "mt" :: "mt_vm" :: !predicates),
+	  "  Assume bytecode multi-threading when doing dependency analyses";
+          
+          "-passopt",
+          Arg.String (fun s -> options := !options @ [s]),
+          "<opt>  Pass this option directly to ocamldoc";
+          
+          "-passrest",
+          Arg.Rest (fun s -> options := !options @ [s]),
+          "  Pass all remaining options directly to ocamldoc";
+          
+	  "-verbose",
+	  Arg.Set verbose,
+	  "  Be verbose\nSTANDARD OPTIONS:";
+        ]
       @
       ( merge_native_arguments
 	  native_spec
@@ -1296,16 +1379,18 @@ let ocamldep () =
       | None -> failwith "Not supported in your configuration: ocamldep"
       | Some s -> s in
 
-  Arg.parse
+  parse_args
     ( [
 	"-syntax", add_syntax_pred,
-                " <p>       Use preprocessor with predicate <p>";
+                "<p>       Use preprocessor with predicate <p>";
 	"-package", add_pkg,
-	         " <p>      Add preprocessor package <p>";
+	         "<p>      Add preprocessor package <p>";
 	"-ppopt", add_pp_opt,
-               " <opt>      Append option <opt> to preprocessor invocation";
+               "<opt>      Append option <opt> to preprocessor invocation";
 	"-passopt", Arg.String (fun s -> pass_options := !pass_options @ [s]),
-                 " <opt>    Pass option <opt> directly to ocamlc/opt/mktop";
+                 "<opt>    Pass option <opt> directly to ocamlc/opt/mktop";
+        "-passrest", Arg.Rest (fun s -> pass_options := !pass_options @ [s]),
+                  "         Pass all remaining options directly";
 	"-native-filter", Arg.Set native_filter,
 	               "    Output only dependencies for native code (implies -native)";
 	"-bytecode-filter", Arg.Set bytecode_filter,
@@ -1380,32 +1465,36 @@ let ocamldep () =
 (************************************************************************)
 
 let ocamlbrowser () =
-  let switches = ref [] in
+  (* let switches = ref [] in *)
   let pass_options = ref [] in
   let add_all = ref false in
 
   let packages = ref [] in
 
+(*
   let add_switch name =
     Arg.Unit (fun () ->
                 switches := name :: !switches;
                 pass_options := !pass_options @ [name]) in
+ *)
   let add_spec_fn name s =
     pass_options := !pass_options @ [name; s] in
-  let add_spec name = Arg.String (add_spec_fn name) in
+(* let add_spec name = Arg.String (add_spec_fn name) in *)
   let add_pkg =
     Arg.String (fun s -> packages := !packages @ (Fl_split.in_words s)) in
 
-  Arg.parse
+  parse_args
       [
 	"-I", Arg.String (fun s -> add_spec_fn "-I" (slashify(resolve_path s))),
-           " <dir>          Add <dir> to the list of include directories";
+           "<dir>          Add <dir> to the list of include directories";
 	"-all", Arg.Set add_all,
 	     "              Add all packages to include path";
 	"-package", add_pkg,
-	         " <p>      Add package <p> to include path";
+	         "<p>      Add package <p> to include path";
 	"-passopt", Arg.String (fun s -> pass_options := !pass_options @ [s]),
-                 " <opt>    Pass option <opt> directly to ocamlbrowser";
+                 "<opt>    Pass option <opt> directly to ocamlbrowser";
+        "-passrest", Arg.Rest (fun s -> pass_options := !pass_options @ [s]),
+                 "          Pass all remaining options directly";
       ]
       (fun s -> raise (Arg.Bad ("Unexpected argument: " ^ s)))
       ("usage: ocamlfind ocamlbrowser [options] file ...");
@@ -1636,6 +1725,7 @@ let install_package () =
   let dll_files = ref [] in
   let nodll_files = ref [] in
   let which = ref Auto in
+  let add_files = ref false in
   let optional = ref false in
   let patches = ref [] in
 
@@ -1654,6 +1744,8 @@ let install_package () =
            "              The following files are DLLs";
       "-nodll", Arg.Unit (fun () -> which := No_dll),
              "            The following files are not DLLs";
+      "-add", Arg.Unit (fun () -> add_files := true),
+           "              Add files to the package";
       "-optional", Arg.Set optional,
                 "         The following files are optional";
       "-patch-version", Arg.String (fun s -> patches := !patches @ [`Version s]),
@@ -1665,7 +1757,7 @@ let install_package () =
     ] in
   let errmsg = "usage: ocamlfind install [options] <package_name> <file> ..." in
 
-  Arg.parse
+  parse_args
         keywords
 	(fun s ->
 	   if !pkgname = ""
@@ -1716,7 +1808,19 @@ let install_package () =
 	nodll_list
     with
       | Not_found ->
-	  failwith "The META file is missing" in
+	  if !add_files then (
+	    let m1 = Filename.concat !metadir meta_dot_pkg in
+	    let m2 = Filename.concat pkgdir "META" in
+	    if Sys.file_exists m1 then
+	      m1
+	    else
+	      if Sys.file_exists m2 then
+		m2
+	      else
+		failwith "Cannot find META in package dir"
+	  )
+	  else
+	    failwith "The META file is missing" in
 
   let meta_pkg =
     let f = open_in meta_name in
@@ -1731,13 +1835,14 @@ let install_package () =
 	  failwith ("Cannot parse '" ^ meta_name ^ "': " ^ s)
   in
 
-  (* Check for frequent reasons why installation can go wrong *)
-  if Sys.file_exists (Filename.concat !metadir meta_dot_pkg) then
-    failwith ("Package " ^ !pkgname ^ " is already installed\n - (file " ^ Filename.concat !metadir meta_dot_pkg ^ " already exists)");
-
-  if Sys.file_exists (Filename.concat pkgdir "META") then
-    failwith ("Package " ^ !pkgname ^ " is already installed\n - (file " ^ pkgdir ^ "/META already exists)");
+  if not !add_files then (
+    (* Check for frequent reasons why installation can go wrong *)
+    if Sys.file_exists (Filename.concat !metadir meta_dot_pkg) then
+      failwith ("Package " ^ !pkgname ^ " is already installed\n - (file " ^ Filename.concat !metadir meta_dot_pkg ^ " already exists)");
 
+    if Sys.file_exists (Filename.concat pkgdir "META") then
+      failwith ("Package " ^ !pkgname ^ " is already installed\n - (file " ^ pkgdir ^ "/META already exists)");
+  );
   List.iter
     (fun f ->
        let f' = Filename.concat pkgdir f in
@@ -1801,10 +1906,12 @@ let install_package () =
       prerr_endline ("Installed " ^ p);
     )
   in
-  if has_metadir then
-    write_meta true !metadir meta_dot_pkg
-  else
-    write_meta false pkgdir "META";
+  if not !add_files then (
+    if has_metadir then
+      write_meta true !metadir meta_dot_pkg
+    else
+      write_meta false pkgdir "META";
+  );
 
   (* Copy the DLLs into the libexec directory if necessary *)
   if have_libexec then begin
@@ -1876,7 +1983,7 @@ let remove_package () =
     ] in
   let errmsg = "usage: ocamlfind remove [options] <package_name>" in
 
-  Arg.parse
+  parse_args
         keywords
 	(fun s ->
 	   if !pkgname = ""
@@ -1980,7 +2087,7 @@ let list_packages() =
     ] in
   let errmsg = "usage: ocamlfind list [options]" in
 
-  Arg.parse
+  parse_args
       keywords
       (fun _ -> Arg.usage keywords errmsg; exit 1)
       errmsg;
@@ -2001,7 +2108,7 @@ let print_configuration() =
   let var = ref None in
   let errmsg = "usage: ocamlfind printconf (conf|path|destdir|metadir|stdlib|ldconf)" in
 
-  Arg.parse
+  parse_args
         []
 	(fun s ->
 	   if !var <> None then raise(Arg.Bad "Unexpected argument");
@@ -2079,6 +2186,7 @@ let rec select_mode () =
     | ("remove"|"-remove")                 -> incr Arg.current; M_remove
     | ("ocamlc"|"-ocamlc"|"c")             -> incr Arg.current; M_compiler "ocamlc"
     | ("ocamlcp"|"-ocamlcp"|"cp")          -> incr Arg.current; M_compiler "ocamlcp"
+    | ("ocamloptp"|"-ocamloptp"|"optp")    -> incr Arg.current; M_compiler "ocamloptp"
     | ("ocamlmklib"|"-ocamlmklib"|"mklib") -> incr Arg.current; M_compiler "ocamlmklib"
     | ("ocamlmktop"|"-ocamlmktop"|"mktop") -> incr Arg.current; M_compiler "ocamlmktop"
     | ("ocamlopt"|"-ocamlopt"|"opt")       -> incr Arg.current; M_compiler "ocamlopt"
@@ -2125,11 +2233,11 @@ let main() =
     | M_remove         -> remove_package ()
     | M_printconf      -> print_configuration ()
     | M_list           -> list_packages()
-    | M_compiler which -> ocamlc which ()
     | M_dep            -> ocamldep()
     | M_browser        -> ocamlbrowser()
     | M_doc            -> ocamldoc()
     | M_call(pkg,cmd)  -> ocamlcall pkg cmd
+    | M_compiler which -> ocamlc which ()
   with
     Usage ->
       prerr_endline "Usage: ocamlfind query        [-help | other options] <package_name> ...";
@@ -2137,7 +2245,10 @@ let main() =
       prerr_endline "   or: ocamlfind ocamlcp      [-help | other options] <file> ...";
       prerr_endline "   or: ocamlfind ocamlmklib   [-help | other options] <file> ...";
       prerr_endline "   or: ocamlfind ocamlmktop   [-help | other options] <file> ...";
-      prerr_endline "   or: ocamlfind ocamlopt     [-help | other options] <file> ...";
+      if Ocaml_args.ocamlopt_spec <> None then
+	prerr_endline "   or: ocamlfind ocamlopt     [-help | other options] <file> ...";
+      if Ocaml_args.ocamloptp_spec <> None then
+	prerr_endline "   or: ocamlfind ocamloptp    [-help | other options] <file> ...";
       prerr_endline "   or: ocamlfind ocamldep     [-help | other options] <file> ...";
       prerr_endline "   or: ocamlfind ocamlbrowser [-help | other options]";
       prerr_endline "   or: ocamlfind ocamldoc     [-help | other options] <file> ...";
