@@ -117,7 +117,7 @@ end
 
 type symbol = {
   mutable sym_pos: int;
-  sym_name: string;
+  mutable sym_name: string;
   mutable value: int32;
   mutable section: [ `Num of int | `Section of section ];
   stype: int;
@@ -135,7 +135,7 @@ and reloc = {
 
 and section = {
   mutable sec_pos: int;
-  sec_name: string;
+  mutable sec_name: string;
   mutable vsize: int32;
   mutable vaddress: int32;
   mutable data:
@@ -145,7 +145,7 @@ and section = {
     | `Sxdata of symbol array
 ];
   mutable relocs: reloc list;
-  sec_opts: int32;
+  mutable sec_opts: int32;
 }
 
 type coff = {
@@ -568,7 +568,8 @@ module Section = struct
       else delayed_ptr oc
           (fun () ->
             if many_relocs then begin
-              emit_int32 oc (Int32.of_int nrelocs);
+              emit_int32 oc (Int32.of_int (nrelocs + 1));
+              (* +1 because this slot counts as well! *)
               emit_int32 oc 0l;
               emit_int16 oc 0
             end;
