@@ -10,8 +10,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id$ *)
-
 (* "Package" a set of .cmo files into one .cmo file having the
    original compilation units as sub-modules. *)
 
@@ -21,7 +19,7 @@ type error =
     Forward_reference of string * Ident.t
   | Multiple_definition of string * Ident.t
   | Not_an_object_file of string
-  | Illegal_renaming of string * string
+  | Illegal_renaming of string * string * string
   | File_not_found of string
 
 exception Error of error
