@@ -145,6 +145,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
   DELETE_RULE Gram meth_list: meth_decl; opt_dot_dot END;
   DELETE_RULE Gram expr: "let"; opt_rec; binding; "in"; SELF END;
   DELETE_RULE Gram expr: "let"; "module"; a_UIDENT; module_binding0; "in"; SELF END;
+  DELETE_RULE Gram expr: "let"; "open"; "!"; module_longident; "in"; SELF END;      
   DELETE_RULE Gram expr: "let"; "open"; module_longident; "in"; SELF END;
   DELETE_RULE Gram expr: "fun"; "["; LIST0 match_case0 SEP "|"; "]" END;
   DELETE_RULE Gram expr: "if"; SELF; "then"; SELF; "else"; SELF END;
@@ -190,7 +191,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
   clear package_type;
   clear top_phrase;
 
-  EXTEND Gram
+  let apply ()  = EXTEND Gram
     GLOBAL:
       a_CHAR a_FLOAT a_INT a_INT32 a_INT64 a_LABEL a_LIDENT
       a_NATIVEINT a_OPTLABEL a_STRING a_UIDENT a_ident
@@ -254,6 +255,8 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
               | _ -> <:str_item< value $rec:r$ $bi$ >> ]
           | "let"; "module"; m = a_UIDENT; mb = module_binding0; "in"; e = expr ->
               <:str_item< let module $m$ = $mb$ in $e$ >>
+          | "let"; "open"; "!"; i = module_longident; "in"; e = expr ->
+              <:str_item< let open! $id:i$ in $e$ >>
           | "let"; "open"; i = module_longident; "in"; e = expr ->
               <:str_item< let open $id:i$ in $e$ >>
       ] ]
@@ -272,6 +275,8 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
         | "let"; "module"; m = a_UIDENT; mb = module_binding0; "in";
           e = expr LEVEL ";" ->
             <:expr< let module $m$ = $mb$ in $e$ >>
+        | "let"; "open"; "!"; i = module_longident; "in"; e = expr LEVEL ";" ->
+            <:expr< let open! $id:i$ in $e$ >>
         | "let"; "open"; i = module_longident; "in"; e = expr LEVEL ";" ->
             <:expr< let open $id:i$ in $e$ >>
         | "function"; a = match_case ->
@@ -528,8 +533,18 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
     meth_list:
       [ [ m = meth_decl -> (m, <:row_var_flag<>>) ] ];
     comma_ctyp_app:
-      [ [ t1 = ctyp; ","; t2 = SELF -> fun acc -> t2 <:ctyp< $acc$ $t1$ >>
-        | t = ctyp -> fun acc -> <:ctyp< $acc$ $t$ >>
+      [ [ t1 = ctyp; ","; t2 = SELF ->
+        fun acc ->
+          let loc1 = Ast.loc_of_ctyp t1 in
+          let loc2 = Ast.loc_of_ctyp acc in
+          let _loc = Loc.smart_merge loc1 loc2 in
+          t2 <:ctyp< $acc$ $t1$ >>
+        | t = ctyp ->
+            fun acc ->
+              let loc1 = Ast.loc_of_ctyp t in
+              let loc2 = Ast.loc_of_ctyp acc in
+              let _loc = Loc.smart_merge loc1 loc2 in
+              <:ctyp< $acc$ $t$ >>
       ] ]
     ;
     star_ctyp:
@@ -706,7 +721,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
         | `EOI -> None
       ] ]
     ;
-  END;
+  END in apply ();
 
   (* Some other DELETE_RULE are before the grammar *)
   DELETE_RULE Gram module_longident_with_app: "("; SELF; ")" END;
