@@ -370,9 +370,9 @@ New syntax:\
     parser
     [ [: `((KEYWORD "(", _) as tok); xs :] ->
         match xs with parser
-        [ [: `(KEYWORD ("or"|"mod"|"land"|"lor"|"lxor"|"lsl"|"lsr"|"asr" as i), _loc);
-             `(KEYWORD ")", _); xs :] ->
-                [: `(LIDENT i, _loc); infix_kwds_filter xs :]
+        [ [: `(KEYWORD ("or"|"mod"|"land"|"lor"|"lxor"|"lsl"|"lsr"|"asr"|"*" as i)  , _loc);
+             `(KEYWORD ")" , _); xs :] ->
+               [: `(LIDENT i, _loc); infix_kwds_filter xs :]
         | [: xs :] ->
                 [: `tok; infix_kwds_filter xs :] ]
     | [: `x; xs :] -> [: `x; infix_kwds_filter xs :] ];
@@ -400,7 +400,7 @@ New syntax:\
     parser [: a = symb; s :] -> kont a s
   end;
 
-  EXTEND Gram
+  let apply () = EXTEND Gram
     GLOBAL:
       a_CHAR a_FLOAT a_INT a_INT32 a_INT64 a_LABEL a_LIDENT rec_binding_quot
       a_NATIVEINT a_OPTLABEL a_STRING a_UIDENT a_ident
@@ -473,7 +473,11 @@ New syntax:\
             <:str_item< module rec $mb$ >>
         | "module"; "type"; i = a_ident; "="; mt = module_type ->
             <:str_item< module type $i$ = $mt$ >>
-        | "open"; i = module_longident -> <:str_item< open $i$ >>
+        | "open"; "!"; i = module_longident -> Ast.StOpn _loc Ast.OvOverride i                 
+        | "open"; i = module_longident ->
+            Ast.StOpn _loc Ast.OvNil i 
+            (* <:str_item< open $i$ >> *)
+
         | "type"; td = type_declaration ->
             <:str_item< type $td$ >>
         | value_let; r = opt_rec; bi = binding ->
@@ -551,7 +555,8 @@ New syntax:\
             <:sig_item< module type $i$ = $mt$ >>
         | "module"; "type"; i = a_ident ->
             <:sig_item< module type $i$ >>
-        | "open"; i = module_longident -> <:sig_item< open $i$ >>
+        | "open"; i = module_longident ->
+            <:sig_item< open $i$ >>
         | "type"; t = type_declaration ->
             <:sig_item< type $t$ >>
         | value_val; i = a_LIDENT; ":"; t = ctyp ->
@@ -601,6 +606,9 @@ New syntax:\
             <:expr< let $rec:r$ $bi$ in $x$ >>
         | "let"; "module"; m = a_UIDENT; mb = module_binding0; "in"; e = SELF ->
             <:expr< let module $m$ = $mb$ in $e$ >>
+
+        | "let"; "open"; "!"; i = module_longident; "in"; e = SELF ->
+            <:expr< let open! $id:i$ in $e$>>
         | "let"; "open"; i = module_longident; "in"; e = SELF ->
             <:expr< let open $id:i$ in $e$ >>
         | "fun"; "["; a = LIST0 match_case0 SEP "|"; "]" ->
@@ -696,7 +704,7 @@ New syntax:\
         | s = a_STRING -> <:expr< $str:s$ >>
         | s = a_CHAR -> <:expr< $chr:s$ >>
         | i = TRY module_longident_dot_lparen; e = sequence; ")" ->
-            <:expr< let open $i$ in $e$ >>
+            <:expr< let open $i$ in $e$ >> 
         | i = TRY val_longident -> <:expr< $id:i$ >>
         | "`"; s = a_ident -> <:expr< ` $s$ >>
         | "["; "]" -> <:expr< [] >>
@@ -769,6 +777,9 @@ New syntax:\
             k <:expr< let module $m$ = $mb$ in $e$ >>
         | "let"; "module"; m = a_UIDENT; mb = module_binding0; ";"; el = SELF ->
             <:expr< let module $m$ = $mb$ in $mksequence _loc el$ >>
+
+        | "let"; "open"; "!"; i = module_longident; "in"; e = SELF ->
+            <:expr< let open! $id:i$ in $e$ >>
         | "let"; "open"; i = module_longident; "in"; e = SELF ->
             <:expr< let open $id:i$ in $e$ >>
         | `ANTIQUOT ("list" as n) s -> <:expr< $anti:mk_anti ~c:"expr;" n s$ >>
@@ -1918,7 +1929,7 @@ New syntax:\
     expr_eoi:
       [ [ x = expr; `EOI -> x ] ]
     ;
-  END;
+  END in apply ();
 
 end;
 
