@@ -11,11 +11,10 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id$ *)
-
 (************************ Simple pattern matching **********************)
 
 open Parser_aux
 
 val pattern_matching :
-  pattern -> Debugcom.remote_value -> Typedtree.type_expr -> (string * Debugcom.remote_value * Typedtree.type_expr) list;;
+  pattern -> Debugcom.remote_value -> Typedtree.type_expr ->
+    (string * Debugcom.remote_value * Typedtree.type_expr) list;;
