@@ -11,8 +11,6 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id$ */
-
 #include <mlvalues.h>
 #include <memory.h>
 #include "unixsupport.h"
@@ -23,7 +21,7 @@ char ** cstringvect(value arg)
   mlsize_t size, i;
 
   size = Wosize_val(arg);
-  res = (char **) stat_alloc((size + 1) * sizeof(char *));
+  res = (char **) caml_stat_alloc((size + 1) * sizeof(char *));
   for (i = 0; i < size; i++) res[i] = String_val(Field(arg, i));
   res[size] = NULL;
   return res;
